import React, { createContext, useState, useContext, useEffect } from 'react';
import { instanceOf, func } from 'prop-types';
import auth0 from 'auth0-js';
import startsWith from 'lodash/startsWith';
import Lockr from 'lockr';
import { v4 } from 'uuid';

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function resolveUri(uri) {
  return startsWith(uri, '/') ? `${window.location.origin}${uri}` : uri;
}

class Auth0Client {
  constructor(_props) {
    _defineProperty(this, "isAuthenticated", () => {
      const currentAuth = Lockr.get(this.props.storageKey);
      return new Date().getTime() < ((currentAuth === null || currentAuth === void 0 ? void 0 : currentAuth.expiresAt) || 0);
    });

    _defineProperty(this, "setSession", authResult => {
      var _authResult$idTokenPa;

      const expiresAt = (authResult === null || authResult === void 0 ? void 0 : (_authResult$idTokenPa = authResult.idTokenPayload) === null || _authResult$idTokenPa === void 0 ? void 0 : _authResult$idTokenPa.exp) * 1000;
      Lockr.set(this.props.storageKey, { ...authResult,
        expiresAt
      });
    });

    _defineProperty(this, "login", props => {
      const nonce = v4();
      Lockr.set(this.props.nonceKey, nonce);
      this.client.authorize({
        nonce,
        state: nonce,
        ...props
      });
    });

    _defineProperty(this, "logout", props => {
      Lockr.rm(this.props.storageKey);
      Lockr.rm(this.props.nonceKey);
      this.client.logout({
        returnTo: resolveUri(this.props.logoutRedirectUri),
        clientID: this.props.clientID,
        ...props
      });
    });

    _defineProperty(this, "authenticate", () => new Promise((resolve, reject) => {
      if (this.isAuthenticated()) return resolve(Lockr.get(this.props.storageKey));
      const nonce = Lockr.get(this.props.nonceKey);
      return this.client.parseHash({
        hash: window.location.hash,
        nonce,
        state: nonce
      }, (err, authResult) => {
        if (authResult && authResult.accessToken && authResult.idToken) {
          this.setSession(authResult);
          resolve(authResult);
        } else {
          reject(err || 'failed to authenticate');
        }
      });
    }));

    _defineProperty(this, "renewSession", async () => {
      await this.authenticate().catch(() => ({}));
      return new Promise((resolve, reject) => {
        const authSession = Lockr.get(this.props.storageKey);
        if (!authSession) return resolve(); // We resolve if a renewable session does not exist

        if (this.isAuthenticated()) return resolve(authSession);
        const nonce = Lockr.get(this.props.nonceKey);
        return this.client.checkSession({
          nonce
        }, (err, authResult) => {
          if (authResult && authResult.accessToken && authResult.idToken) {
            this.setSession(authResult);
            resolve(authResult);
          } else {
            Lockr.rm(this.props.storageKey);
            Lockr.rm(this.props.nonceKey);
            reject(err || 'failed to renew session');
          }
        });
      });
    });

    this.props = {
      redirectUri: '/authorize',
      responseType: 'token id_token',
      scope: 'openid',
      logoutRedirectUri: '/login',
      storageKey: 'auth',
      nonceKey: 'nonce',
      ..._props
    };
    const {
      domain,
      clientID,
      redirectUri,
      responseType,
      scope,
      audience
    } = this.props;
    this.client = new auth0.WebAuth({
      domain,
      clientID,
      redirectUri: resolveUri(redirectUri),
      responseType,
      scope,
      audience
    });
  }

}

var Context = /*#__PURE__*/createContext();

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }
function Auth0Provider(props) {
  const {
    client
  } = props;
  const [authenticated, setAuthenticated] = useState(client.isAuthenticated());
  const [error, setError] = useState();
  const [authenticating, setAuthenticating] = useState(false);

  function logout(logoutProps) {
    setAuthenticated(false);
    client.logout(logoutProps);
  }

  async function authenticate() {
    try {
      setError(undefined);
      setAuthenticating(true);
      await client.authenticate();
      setAuthenticated(client.isAuthenticated());
    } catch (err) {
      setError(err);
    } finally {
      setAuthenticating(false);
    }
  }

  return /*#__PURE__*/React.createElement(Context.Provider, _extends({
    value: {
      login: client.login,
      logout,
      authenticate,
      authenticated,
      authenticating,
      error
    }
  }, props));
}
Auth0Provider.propTypes = {
  client: instanceOf(Auth0Client).isRequired
};

function useAuth0() {
  return useContext(Context);
}

function Login(props = {}) {
  const {
    login
  } = useAuth0();
  useEffect(() => {
    login(props);
  }, []);
  return null;
}

function Logout(props = {}) {
  const {
    logout
  } = useAuth0();
  useEffect(() => {
    logout(props);
    props.onDidLogout();
  }, []);
  return null;
}
Logout.propTypes = {
  onDidLogout: func
};
Logout.defaultProps = {
  onDidLogout() {}

};

function useAuthorize() {
  const {
    authenticate,
    authenticated,
    authenticating,
    error
  } = useAuth0();
  useEffect(() => {
    authenticate();
  }, []);
  return [{
    authenticated,
    error
  }, authenticating];
}

function Authorize(props = {}) {
  const [{
    authenticated,
    error
  }, authenticating] = useAuthorize();
  return props.children({
    authenticating,
    authenticated,
    error
  });
}
Authorize.propTypes = {
  children: func
};
Authorize.defaultProps = {
  children() {}

};

export { Authorize, Context, Login, Logout, Auth0Provider as Provider, Auth0Client as WebClient, useAuth0, useAuthorize };
//# sourceMappingURL=index.js.map
